# -*- coding: utf-8 -*-
# gpodder.net API Client
# Copyright (C) 2009-2010 Thomas Perl
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import mygpoclient

from mygpoclient import locator
from mygpoclient import json
from mygpoclient import simple

class ToplistPodcast(object):
    """Container class for a toplist entry

    This class encapsulates the metadata for a podcast
    in the podcast toplist.

    Attributes:
    url - The feed URL of the podcast
    title - The title of the podcast
    description - The description of the podcast
    subscribers - The current subscriber count
    subscribers_last_week - Last week's subscriber count
    """
    REQUIRED_KEYS = ('url', 'title', 'description', \
                     'subscribers', 'subscribers_last_week')

    def __init__(self, url, title, description,
            subscribers, subscribers_last_week):
        self.url = url
        self.title = title
        self.description = description
        self.subscribers = int(subscribers)
        self.subscribers_last_week = int(subscribers_last_week)

    def __eq__(self, other):
        """Test two ToplistPodcast objects for equality

        >>> ToplistPodcast('u', 't', 'd', 10, 12) == ToplistPodcast('u', 't', 'd', 10, 12)
        True
        >>> ToplistPodcast('u', 't', 'd', 10, 12) == ToplistPodcast('a', 'b', 'c', 13, 14)
        False
        >>> ToplistPodcast('u', 't', 'd', 10, 12) == 'x'
        False
        """
        if not isinstance(other, self.__class__):
            return False

        return all(getattr(self, k) == getattr(other, k) \
                for k in self.REQUIRED_KEYS)

    @classmethod
    def from_dict(cls, d):
        for key in cls.REQUIRED_KEYS:
            if key not in d:
                raise ValueError('Missing keys for toplist podcast')

        return cls(*(d.get(k) for k in cls.REQUIRED_KEYS))


class PublicClient(object):
    """Client for the gpodder.net "anonymous" API

    This is the API client implementation that provides a
    pythonic interface to the parts of the gpodder.net
    Simple API that don't need user authentication.
    """
    FORMAT = 'json'

    def __init__(self, host=mygpoclient.HOST, client_class=json.JsonClient):
        """Creates a new Public API client

        The parameter host is optional and defaults to
        the main webservice.

        The parameter client_class is optional and should
        not need to be changed in normal use cases. If it
        is changed, it should provide the same interface
        as the json.JsonClient class in mygpoclient.
        """
        self._locator = locator.Locator(None, host)
        self._client = client_class(None, None)

    def get_toplist(self, count=mygpoclient.TOPLIST_DEFAULT):
        """Get a list of most-subscribed podcasts

        Returns a list of ToplistPodcast objects.

        The parameter "count" is optional and describes
        the amount of podcasts that are returned. The
        default value is 50, the minimum value is 1 and
        the maximum value is 100.
        """
        uri = self._locator.toplist_uri(count, self.FORMAT)
        return [ToplistPodcast.from_dict(x) for x in self._client.GET(uri)]

    def search_podcasts(self, query):
        """Search for podcasts on the webservice

        Returns a list of simple.Podcast objects.

        The parameter "query" specifies the search
        query as a string.
        """
        uri = self._locator.search_uri(query, self.FORMAT)
        return [simple.Podcast.from_dict(x) for x in self._client.GET(uri)]




# -*- coding: utf-8 -*-
# gpodder.net API Client
# Copyright (C) 2009-2010 Thomas Perl
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from mygpoclient import public
from mygpoclient import simple
from mygpoclient import testing

import unittest

class Test_ToplistPodcast(unittest.TestCase):
    def test_toplistPodcastFromDict_raisesValueError_missingKey(self):
        d = {
                'url': 'http://example.org/feeds/podcast.rss',
                'title': 'My Example Podcast Feed',
        }
        self.assertRaises(ValueError,
                public.ToplistPodcast.from_dict, d)


class Test_PublicClient(unittest.TestCase):
    TOPLIST_JSON = """
    [{"url": "http://twit.tv/node/4350/feed",
     "title": "FLOSS Weekly",
     "description": "Free, Libre and Open Source Software with Leo.",
     "subscribers": 4711,
     "subscribers_last_week": 4700
    },
    {"url": "http://feeds.feedburner.com/LinuxOutlaws",
     "title": "The Linux Outlaws",
     "description": "A podcast about Linux with Dan and Fab.",
     "subscribers": 1337,
     "subscribers_last_week": 1330
    }]
    """
    TOPLIST = [
            public.ToplistPodcast('http://twit.tv/node/4350/feed',
                'FLOSS Weekly',
                'Free, Libre and Open Source Software with Leo.',
                4711, 4700),
            public.ToplistPodcast('http://feeds.feedburner.com/LinuxOutlaws',
                'The Linux Outlaws',
                'A podcast about Linux with Dan and Fab.',
                1337, 1330),
    ]
    SEARCHRESULT_JSON = """
    [{"url": "http://twit.tv/node/4350/feed",
      "title": "FLOSS Weekly",
      "description": "Free, Libre and Open Source Software with Leo."
    },
    {"url": "http://feeds.feedburner.com/LinuxOutlaws",
      "title": "The Linux Outlaws",
      "description": "A podcast about Linux with Dan and Fab."
    }]
    """
    SEARCHRESULT = [
            simple.Podcast('http://twit.tv/node/4350/feed',
                'FLOSS Weekly',
                'Free, Libre and Open Source Software with Leo.'),
            simple.Podcast('http://feeds.feedburner.com/LinuxOutlaws',
                'The Linux Outlaws',
                'A podcast about Linux with Dan and Fab.'),
    ]

    def setUp(self):
        self.fake_client = testing.FakeJsonClient()
        self.client = public.PublicClient(client_class=self.fake_client)

    def test_getToplist(self):
        self.fake_client.response_value = self.TOPLIST_JSON
        result = self.client.get_toplist()
        self.assertEquals(result, self.TOPLIST)
        self.assertEquals(len(self.fake_client.requests), 1)

    def test_searchPodcasts(self):
        self.fake_client.response_value = self.SEARCHRESULT_JSON
        result = self.client.search_podcasts('wicked')
        self.assertEquals(result, self.SEARCHRESULT)
        self.assertEquals(len(self.fake_client.requests), 1)




# -*- coding: utf-8 -*-
# gpodder.net API Client
# Copyright (C) 2009-2010 Thomas Perl
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from mygpoclient import simple
from mygpoclient import testing

import unittest

class Test_Podcast(unittest.TestCase):
    def test_podcastFromDict_raisesValueError_missingKey(self):
        self.assertRaises(ValueError,
                simple.Podcast.from_dict, {'url': 'a', 'title': 'b'})

class Test_SimpleClient(unittest.TestCase):
    USERNAME = 'a'
    PASSWORD = 'b'
    DEVICE_NAME = 'x'
    SUBSCRIPTIONS = [
            'http://lugradio.org/episodes.rss',
            'http://feeds2.feedburner.com/LinuxOutlaws',
    ]
    SUBSCRIPTIONS_JSON = """
      ["http://lugradio.org/episodes.rss",
       "http://feeds2.feedburner.com/LinuxOutlaws"]
    """
    SUGGESTIONS = [
      simple.Podcast('http://twit.tv/node/4350/feed',
                     'FLOSS Weekly',
                     'Free, Libre and Open Source Software with Leo.'),
      simple.Podcast('http://feeds.feedburner.com/LinuxOutlaws',
                     'The Linux Outlaws',
                     'A podcast about Linux with Dan and Fab.'),
    ]
    SUGGESTIONS_JSON = """
      [{"url": "http://twit.tv/node/4350/feed",
       "title": "FLOSS Weekly",
       "description": "Free, Libre and Open Source Software with Leo."
      },
      {"url": "http://feeds.feedburner.com/LinuxOutlaws",
       "title": "The Linux Outlaws",
       "description": "A podcast about Linux with Dan and Fab."
      }]
    """

    def setUp(self):
        self.fake_client = testing.FakeJsonClient()
        self.client = simple.SimpleClient(self.USERNAME, self.PASSWORD,
                client_class=self.fake_client)

    def test_putSubscriptions(self):
        self.fake_client.response_value = ''
        result = self.client.put_subscriptions(self.DEVICE_NAME, self.SUBSCRIPTIONS)
        self.assertEquals(result, True)
        self.assertEquals(len(self.fake_client.requests), 1)

    def test_getSubscriptions(self):
        self.fake_client.response_value = self.SUBSCRIPTIONS_JSON
        subscriptions = self.client.get_subscriptions(self.DEVICE_NAME)
        self.assertEquals(subscriptions, self.SUBSCRIPTIONS)
        self.assertEquals(len(self.fake_client.requests), 1)

    def test_getSuggestions(self):
        self.fake_client.response_value = self.SUGGESTIONS_JSON
        suggestions = self.client.get_suggestions(50)
        self.assertEquals(suggestions, self.SUGGESTIONS)
        self.assertEquals(len(self.fake_client.requests), 1)




# -*- coding: utf-8 -*-
# gpodder.net API Client
# Copyright (C) 2009-2010 Thomas Perl
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from distutils.core import setup

import glob

setup(name='mygpoclient',
      version='1.3',
      description='Library for accessing gpodder.net web services.',
      author='Thomas Perl',
      author_email='thp@gpodder.org',
      url='http://thpinfo.com/2010/mygpoclient/',
      packages=['mygpoclient'],
      scripts=glob.glob('scripts/*'))

